# Generated by roxygen2: do not edit by hand

S3method(crprep,default)
S3method(expand.covs,default)
S3method(expand.covs,msdata)
S3method(plot,Cuminc)
S3method(plot,MarkovTest)
S3method(plot,msfit)
S3method(plot,probtrans)
S3method(print,Cuminc)
S3method(print,MarkovTest)
S3method(print,msdata)
S3method(print,summary.msfit)
S3method(print,summary.probtrans)
S3method(summary,Cuminc)
S3method(summary,msfit)
S3method(summary,probtrans)
export(Cuminc)
export(ELOS)
export(LMAJ)
export(MarkovTest)
export(absorbing)
export(crprep)
export(cutLMms)
export(etm2msdata)
export(events)
export(expand.covs)
export(haz_function)
export(is.circular)
export(modify_transMat)
export(msboot)
export(msboot.relsurv)
export(msboot.relsurv.boot)
export(msdata2etm)
export(msfit)
export(msfit.relsurv)
export(msprep)
export(mssample)
export(optimal_weights_matrix)
export(optimal_weights_multiple)
export(paths)
export(probtrans)
export(redrank)
export(to.trans2)
export(tra2trans)
export(trans.comprisk)
export(trans.illdeath)
export(trans2Q)
export(trans2tra)
export(transMat)
export(varHaz.fixed)
export(vis.mirror.pt)
export(vis.multiple.pt)
export(xsect)
import(survival)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,`:=`)
importFrom(graphics,box)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lattice,xyplot)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,delete.response)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,terms)
importFrom(stats,time)
importFrom(stats,weighted.mean)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(mstate, .registration=TRUE)
