# mstate

<!-- badges: start -->
[![](https://www.r-pkg.org/badges/version/mstate?color=orange)](https://cran.r-project.org/package=mstate)
[![](http://cranlogs.r-pkg.org/badges/last-month/mstate?color=gray)](https://cran.r-project.org/package=mstate)
[![](http://cranlogs.r-pkg.org/badges/grand-total/mstate?color=blue)](https://cran.r-project.org/package=mstate)
[![](https://img.shields.io/badge/doi-10.1002/sim.2712-yellow.svg)](https://doi.org/10.1002/sim.2712)
<!-- badges: end -->

## Installation

You can install the released version of mstate from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("mstate")
```
Alternatively, you can install the development version from GitHub:

``` r
devtools::install_github("hputter/mstate")
```
